package CH.ifa.draw.application;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.IOHelper;

import CH.ifa.draw.application.MenuManager.SeparatorFactory;

import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingTypeManager;
import CH.ifa.draw.framework.DrawingTypeManagerListener;

import CH.ifa.draw.io.DrawingFileHelper;
import CH.ifa.draw.io.ExportHolderImpl;
import CH.ifa.draw.io.ImportHolderImpl;
import CH.ifa.draw.io.SimpleFileFilter;

import CH.ifa.draw.standard.NullDrawing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import java.io.File;

import java.net.URL;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Map;
import java.util.Vector;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;


public class FileMenuCreator {
    private JMenuItem defaultMenuItem = null;
    private DrawingTypeManagerListener typeListener = null;

    public Collection createMenus(ImportHolderImpl importHolder,
                                  ExportHolderImpl exportHolder) {
        Vector result = new Vector();
        SeparatorFactory sepFac = new SeparatorFactory("ch.ifa.draw");

        if (defaultMenuItem == null) {
            defaultMenuItem = DrawApplication.createMenuItem("New "
                                                             + getTypeManager()
                                                                   .getDefaultFileFilter()
                                                                   .getDescription(),
                                                             KeyEvent.VK_N,
                                                             new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        DrawApplication app = DrawPlugin.getGui();
                        app.promptNew();
                    }
                });
            typeListener = new DrawingTypeManagerListener() {
                    public void typeRegistered(String name,
                                               SimpleFileFilter filter) {
                        // Not of interest.
                    }

                    public void defaultTypeChanged(SimpleFileFilter filter) {
                        defaultMenuItem.setText("New "
                                                + filter.getDescription());
                    }
                };
            getTypeManager().addListener(typeListener);
        }
        result.add(defaultMenuItem);

        result.add(DrawApplication.createMenuItem("New Drawing...",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    promptChooseNew();
                }
            }));
        result.add(sepFac.createSeparator());

        result.add(DrawApplication.createMenuItem("Open URL", KeyEvent.VK_U,
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    promptOpenURL();
                }
            }));

        result.add(DrawApplication.createMenuItem("Open Drawing...",
                                                  KeyEvent.VK_O,
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    promptOpen(getIOHelper().getFileFilter());
                }
            }));


        result.add(DrawApplication.createMenuItem("Insert Drawing...",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    app.promptInsert();
                }
            }));

        result.add(DrawApplication.createMenuItem("Save Drawing",
                                                  KeyEvent.VK_S,
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    Drawing drawing = app.drawing();
                    if (!(drawing instanceof NullDrawing)) {
                        app.saveDrawing(drawing);
                    }
                }
            }));

        result.add(DrawApplication.createMenuItem("Save Drawing As...",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    Drawing drawing = app.drawing();
                    if (!(drawing instanceof NullDrawing)
                                && app.promptSaveAs(drawing)) {
                        app.saveDrawing(drawing);
                    }
                }
            }));

        result.add(DrawApplication.createMenuItem("Save All Drawings",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    Enumeration drawings = app.drawings();
                    while (drawings.hasMoreElements()) {
                        Drawing drawing = (Drawing) drawings.nextElement();
                        if (drawing.getFilename() == null) {
                            app.showDrawingViewContainer(drawing);
                            if (!app.promptSaveAs(drawing)) {
                                return;
                            }
                        }
                    }
                    drawings = app.drawings();
                    while (drawings.hasMoreElements()) {
                        app.saveDrawing((Drawing) drawings.nextElement());
                    }
                }
            }));

        result.add(DrawApplication.createMenuItem("Close Drawing",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    if (app.fViewContainer != null) {
                        app.closeViewContainer(app.fViewContainer);
                    }
                }
            }));

        result.add(sepFac.createSeparator());


        //		menu.add(app.createMenuItem("Close Frame",
        //		   new ActionListener() {
        //			  public void actionPerformed(ActionEvent event) {
        //				fViewContainer.hide();
        //				   fViewContainer.dispose();
        //			  }
        //		   }
        //				));
        DrawPlugin plugin = DrawPlugin.getCurrent();
        result.add(importHolder.getImportMenu());
        result.add(exportHolder.getExportMenu());

        result.add(sepFac.createSeparator());


        result.add(DrawApplication.createMenuItem("Print...", KeyEvent.VK_P,
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    app.print();
                }
            }));

        result.add(sepFac.createSeparator());

        //        app.importMenu = new CommandMenu("Import");
        //        app.exportMenu = new CommandMenu("Export");
        //        menu.add(app.exportMenu);
        //        menu.add(app.importMenu);
        //        menu.addSeparator();
        result.add(DrawApplication.createMenuItem("Exit",
                                                  new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    DrawApplication app = DrawPlugin.getGui();
                    app.requestClose();
                }
            }));

        return result;
    }


    /**
     * Creates a new drawing view with a new empty drawing of choosable
     * type. The user is presented with all known drawing types.
     * @return the created drawing. Returns <code>null</code>, if the
     *         dialog was cancelled by the user.
     **/
    public Drawing promptChooseNew() {
        Map drawingTypes = getTypeManager().getDrawingTypes();
        Collection typeNames = drawingTypes.keySet();
        String[] types = (String[]) typeNames.toArray(new String[typeNames.size()]);
        String[] descriptions = new String[types.length];
        for (int i = 0; i < types.length; i++) {
            descriptions[i] = ((FileFilter) drawingTypes.get(types[i]))
                              .getDescription();
        }
        NewDrawingDialog dialog = new NewDrawingDialog(DrawPlugin.getGui()
                                                                 .getFrame(),
                                                       descriptions, 0);
        int result = dialog.showDialog();
        if (result >= 0 && result < types.length) {
            Drawing drawing = DrawingTypeManager.getDrawingFromName(types[result]);
            DrawPlugin.getGui().openDrawing(drawing);
            return drawing;
        }
        return null;
    }

    /**
     * Shows a file dialog and opens a drawing.%%%%%
     */
    public void promptOpen(FileFilter ff) {
        File[] files = getIOHelper().getLoadPath(null, ff, true);
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (ff instanceof SimpleFileFilter) {
                    files[i] = DrawingFileHelper.checkAndAddExtension(files[i],
                                                                      (SimpleFileFilter) ff);
                }
                getIOHelper().loadAndOpenDrawing(files[i]);
            }
        }
    }

    void promptOpenURL() {
        String header =
            "Type in URL:\n Accepted protocols are: http, file, jar.\n"
            + "If not specified I will try to use http.\n"
            + "Examples:\nhttp://my.server.tld/referencenet.rnw.\n"
            + " file:///my/path/to/nets/net.rnw.\n"
            + "jar:file:///my/path/to/jar/the.jar!/path/to/file/net.rnw";
        String in = JOptionPane.showInputDialog(DrawPlugin.getGui().getFrame(),
                                                header);
        if (in != null) {
            // this is maybe to restrictive but rather be on the safe side
            if (!in.startsWith("jar:") && !in.startsWith("file://")) {
                if (!in.startsWith("http://")) {
                    in = "http://" + in; // if none of the ones above, try http
                }
            }

            //System.out.println(in);
            try {
                URL url = new URL(in);
                System.out.println(url);
                getIOHelper().loadAndOpenDrawing(url);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /* end jhotdraw file menu */
    private IOHelper getIOHelper() {
        return DrawPlugin.getCurrent().getIOHelper();
    }

    private DrawingTypeManager getTypeManager() {
        return DrawingTypeManager.getInstance();
    }
}