/*
 * @(#)ToolButton.java 5.1
 *
 */
package CH.ifa.draw.standard;

import CH.ifa.draw.framework.Tool;

import CH.ifa.draw.util.PaletteListener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.File;

import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;


/**
 * A PaletteButton that is associated with a tool.
 * @see Tool
 */
public class ToolButton implements ActionListener, MouseListener {
    private String fName;
    private Tool fTool;
    private JToggleButton fButton;
    private PaletteListener fListener;

    public ToolButton(PaletteListener listener, String iconName, String name,
                      Tool tool) {
        ImageIcon[] icons = createIconsByGuess(iconName);
        createButton(listener, icons[0], icons[1], name, tool);
    }

    public ToolButton(PaletteListener listener, Icon icon, Icon selectedIcon,
                      String name, Tool tool) {
        createButton(listener, icon, selectedIcon, name, tool);
    }

    private void createButton(PaletteListener listener, Icon icon,
                              Icon selectedIcon, String name, Tool tool) {
        fButton = new JToggleButton();
        fButton.setIcon(icon);
        if (selectedIcon != null) {
            fButton.setSelectedIcon(selectedIcon);
        }
        fButton.addMouseListener(this);
        fButton.addActionListener(this);
        fButton.setToolTipText(fName);
        fTool = tool;
        fName = name;
        fListener = listener;
    }

    public Tool tool() {
        return fTool;
    }

    public String name() {
        return fName;
    }

    public Object attributeValue() {
        return tool();
    }

    public AbstractButton button() {
        return fButton;
    }

    public void mouseClicked(MouseEvent e) {
        boolean doubleClick = (e.getClickCount() > 1);
        fListener.paletteUserSelected(this, doubleClick);
    }

    public void mouseEntered(MouseEvent e) {
        fListener.paletteUserOver(this, true);
    }

    public void mouseExited(MouseEvent e) {
        fListener.paletteUserOver(this, false);
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public final void actionPerformed(ActionEvent e) {
        fListener.paletteUserSelected(this, false);
    }

    private static ImageIcon[] createIconsByGuess(final String name) {
        ImageIcon[] result = new ImageIcon[2];
        ImageIcon firstGuess;
        try {
            firstGuess = createIcon(name + ".gif");
        } catch (Exception e) {
            System.out.println("Guess for " + name + ".gif failed with " + e);
            firstGuess = null;
        }
        if (firstGuess != null) {
            result[0] = firstGuess;
            result[1] = null;
        } else {
            result[0] = createIcon(name + "1.gif");
            result[1] = createIcon(name + "2.gif");
        }
        return result;
    }

    private static ImageIcon createIcon(String name) {
        //        System.out.println("icon " + name + " "
        //                           + System.getProperty("user.dir"));
        URL url = ToolButton.class.getResource(name);
        if (url == null) {
            File icon = new File(name);
            if (!icon.isFile()) {
                return null;
            }
            try {
                url = icon.toURI().toURL();
            } catch (MalformedURLException e) {
                return null;
            }
        }
        return new ImageIcon(url);
    }
}