/*
 * @(#)CommandMenu.java 5.1
 *
 */
package CH.ifa.draw.util;

import java.awt.Component;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;


/**
 * A Command enabled menu. Selecting a menu item
 * executes the corresponding command.
 *
 * @see Command
 */
public class CommandMenu extends JMenu implements ActionListener {
    public CommandMenu(String name) {
        super(name);
        checkEnabled();
    }

    public JMenuItem add(JMenuItem item) {
        if (item instanceof CommandMenuItem) {
            // addCommandMenuItem((CommandMenuItem) item);
            item.addActionListener(this);
            JMenuItem result = super.add(item);
            checkEnabled();
            return item;
        } else {
            return super.add(item);
        }
    }

    //    private void addCommandMenuItem(CommandMenuItem m) {
    //        m.setFont(getFont());
    //
    //        // m.setName(command.name()); necessary??
    //        m.addActionListener(this);
    //        super.add(m);
    //        checkEnabled();
    //    }


    /**
     * Adds a command to the menu. The item's label is
     * the command's name.
     */
    public synchronized void add(Command command) {
        add(new CommandMenuItem(command));
    }

    /**
     * Adds a command with the given short cut to the menu. The item's label is
     * the command's name.
     */
    public synchronized void add(Command command, int shortcut) {
        add(new CommandMenuItem(command, shortcut));
    }

    public synchronized void remove(Command command) {
        System.out.println("CommandMenu.remove() not implemented");
    }

    public synchronized void remove(MenuItem item) {
        System.out.println("CommandMenu.remove() not implemented");
    }

    /**
     * Changes the enabling/disabling state of a named menu item.
     */
    public synchronized void enable(String name, boolean state) {
        for (int i = 0; i < getItemCount(); i++) {
            JMenuItem item = getItem(i);
            if (name.equals(item.getText())) {
                item.setEnabled(state);
                return;
            }
        }
    }

    public synchronized boolean checkEnabled() {
        boolean menuEnabled = false;
        boolean enabled;

        for (int i = 0; i < getMenuComponentCount(); i++) {
            Component item = getMenuComponent(i);
            if (item == null) {
                return false;
            }
            if (item instanceof CommandMenuItem) {
                enabled = ((CommandMenuItem) item).cmd.isExecutable();
                item.setEnabled(enabled);
                menuEnabled = menuEnabled | enabled;
            } else if (item instanceof CommandMenu) {
                enabled = ((CommandMenu) item).checkEnabled();
                item.setEnabled(enabled);
                menuEnabled = menuEnabled | enabled;
            } else if (item instanceof JSeparator) {
                // Ignore separators.
            } else {
                menuEnabled = menuEnabled | item.isEnabled();
            }
        }
        return menuEnabled;
    }

    /**
     * Executes the command.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source instanceof CommandMenuItem) {
            Command cmd = ((CommandMenuItem) source).cmd;
            cmd.execute();
        }
    }
}