package de.renew.formalism.fs;

import de.uni_hamburg.fs.ConceptImpl;
import de.uni_hamburg.fs.Name;
import de.uni_hamburg.fs.Partition;
import de.uni_hamburg.fs.TypeException;

import de.renew.shadow.ShadowLookup;
import de.renew.shadow.ShadowLookupExtension;
import de.renew.shadow.ShadowLookupExtensionFactory;
import de.renew.shadow.ShadowNetElement;

import java.util.Enumeration;
import java.util.Hashtable;


public class FSShadowLookupExtension implements ShadowLookupExtension {
    // the factory producing this kind of extension
    private static final Factory _factory = new Factory();

    public static FSShadowLookupExtension lookup(ShadowLookup lookup) {
        return (FSShadowLookupExtension) lookup.getShadowLookupExtension(_factory);
    }

    /** Mapping from ShadowConcept to ConceptImpl. */
    private Hashtable conceptMap = new Hashtable();

    /** Mapping from ConceptImpl.Feature to ShadowConcept or ShadowAssoc. */
    private Hashtable appropMap = new Hashtable();

    /** Mapping from ConceptImpls to Partition of Subconcepts. */
    private Hashtable partitionMap = new Hashtable();

    public void setConcept(ShadowConcept shadowConcept, ConceptImpl concept) {
        conceptMap.put(shadowConcept.getName(), concept);
    }

    public ConceptImpl getConcept(ShadowConcept shadowConcept) {
        return (ConceptImpl) conceptMap.get(shadowConcept.getName());
    }

    /*
    public ShadowConcept getShadowConcept(ConceptImpl concept) {
      // use inverse relation:
      Enumeration enum=conceptMap.keys();
      while (enum.hasMoreElements()) {
    ShadowConcept key=(ShadowConcept)enum.nextElement();
    if (conceptMap.get(key)==concept) {
            return key;
    }
      }
      throw new NoSuchElementException();
    }
    */
    public Enumeration allConcepts() {
        return conceptMap.keys(); // an enumeration of Concept Namess
    }

    public void setApprop(ConceptImpl concept, Name feature,
                          ShadowNetElement elem) {
        appropMap.put(concept.getName() + "." + feature, elem);
        //System.out.println("Adding Attribute "+feature+" to concept "+concept.getName());
    }

    public ShadowNetElement getApprop(ConceptImpl concept, Name feature) {
        return (ShadowNetElement) appropMap.get(concept.getName() + "."
                                                + feature);
    }

    public void addToSubPartition(ConceptImpl supi, ConceptImpl sub)
            throws TypeException {
        Partition suppart = (Partition) partitionMap.get(supi);
        if (suppart == null) {
            System.out.println("Creating new sub-partition for "
                               + supi.getName());
            System.out.println("Adding " + sub.getName()
                               + " to sub-partition of " + supi.getName());
            suppart = new Partition(sub);
            partitionMap.put(supi, suppart);
        } else {
            System.out.println("Adding " + sub.getName()
                               + " to sub-partition of " + supi.getName());
            suppart.addConcept(sub);
        }
    }

    public static class Factory implements ShadowLookupExtensionFactory {
        public String getCategory() {
            return "de.renew.formalism.fs";
        }

        public ShadowLookupExtension createExtension() {
            return new FSShadowLookupExtension();
        }
    }
}