package de.renew.gui.fs;

import CH.ifa.draw.framework.Figure;

import de.renew.gui.CPNTextFigure;
import de.renew.gui.FigureCreator;
import de.renew.gui.FigureCreatorHolder;
import de.renew.gui.GuiPlugin;
import de.renew.gui.InscribableFigure;
import de.renew.gui.TextFigureCreator;

import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginException;
import de.renew.plugin.PluginProperties;

import de.renew.remote.ObjectAccessor;
import de.renew.remote.ObjectAccessorImpl;

import java.net.URL;


/**
 * @author joern
 *
 */
public class FSPlugin extends PluginAdapter {
    public FSPlugin(URL location) throws PluginException {
        super(location);
    }

    public FSPlugin(PluginProperties props) {
        super(props);
    }

    public void init() {
        FigureCreatorHolder fc = GuiPlugin.getCurrent().getFigureCreatorHolder();

        // register the FSFigure as a possible token representation
        fc.registerCreator(new FigureCreator() {
                public boolean canCreateFigure(ObjectAccessor token,
                                               boolean expanded) {
                    if (token instanceof ObjectAccessorImpl) {
                        Object obj = ((ObjectAccessorImpl) token).getObject();
                        return expanded
                               && !(obj instanceof de.uni_hamburg.fs.FeatureStructure
                               || obj instanceof de.renew.net.NetInstance
                               || obj instanceof de.renew.util.Value
                               || obj instanceof String);
                    } else {
                        return false;
                    }
                }

                public Figure getTokenFigure(ObjectAccessor token,
                                             boolean expanded) {
                    return new de.renew.gui.fs.FSFigure(((ObjectAccessorImpl) token)
                                                        .getObject());
                }
            });


        //        FigureCreation.registerTextFigureCreator(new fsTextFigureCreator());
    }

    private class FSTextFigureCreator implements TextFigureCreator {
        public boolean canCreateFigure(InscribableFigure figure) {
            if (figure instanceof ConceptConnection
                        || figure instanceof FeatureConnection) {
                return false;
            }
            return true;
        }

        public CPNTextFigure createTextFigure(InscribableFigure figure) {
            return new FSFigure();
        }

        public boolean canCreateDefaultInscription(InscribableFigure figure) {
            //        		System.out.println ("can i inscribe " + figure + "?");
            if (figure instanceof AssocConnection) {
                return true;
            }
            if (figure instanceof FeatureConnection) {
                return true;
            }
            return false;
        }

        public String getDefaultInscription(InscribableFigure figure) {
            if (figure instanceof AssocConnection) {
                AssocConnection assoc = (AssocConnection) figure;
                ConceptFigure concept = (ConceptFigure) assoc.endFigure();
                if (concept == null || "".equals(concept.getName())) {
                    return "relatesTo";
                } else {
                    return concept.getName().toLowerCase();
                }
            }
            if (figure instanceof FeatureConnection) {
                return "feat";
            }
            return "invalid TextFigureCreator: " + getClass();
        }
    }
}