package de.renew.ant;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

import de.renew.gui.CPNDrawing;
import de.renew.gui.ShadowTranslator;

import de.renew.shadow.ShadowCompilerFactory;

import java.io.File;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;


/**
 * This Ant task can transform Renew net drawings
 * (<code>.rnw</code>) into shadow net system files
 * (<code>.sns</code>) suitable for Renew's net loading
 * mechanism. It can operate on a file set and puts each
 * translated file into the directory containing its source
 * drawing.
 *
 * @author "Timo Carl" <6carl@informatik.uni-hamburg.de>
 *
 * @see ShadowTranslator
 **/
public class CreateShadownetsTask extends Task {
    private String compilerName = null;
    private File destdir = null;
    private Vector filesets = new Vector();

    public void setDestdir(File dest) {
        this.destdir = dest;
    }

    public void setCompilerName(String compilerName) {
        this.compilerName = compilerName;
    }

    public void addConfiguredFileset(FileSet fileset) {
        this.filesets.add(fileset);
    }

    public void execute() throws BuildException {
        //disable gui mode of renew
        System.setProperty("de.renew.noGraphics", "true");

        Enumeration enum = filesets.elements();
        while (enum.hasMoreElements()) {
            FileSet fs = (FileSet) enum.nextElement();
            DirectoryScanner ds = fs.getDirectoryScanner(getProject());
            File dir = ds.getBasedir();
            String[] nets = ds.getIncludedFiles();
            ArrayList l = new ArrayList();
            for (int i = 0, n = nets.length; i < n; i++) {
                l.add(new File(dir, nets[i]));
            }
            CPNDrawing[] drawings = ShadowTranslator.readDrawings((File[]) l
                                                                    .toArray(new File[l
                                                                                      .size()]));
            ShadowTranslator.writeShadows(getShadowCompiler(), drawings);
        }
    }

    private ShadowCompilerFactory getShadowCompiler() {
        if (compilerName == null) {
            return null;
        }
        try {
            // We use the standard class loader because the plugin system
            // is not necessarily running.
            Class c = Class.forName(compilerName);
            return (ShadowCompilerFactory) c.newInstance();
        } catch (Exception e) {
            System.out.println("Could not create ShadowNetCompiler. Reason:\n"
                               + e);
            return null;
        }
    }
}