package de.renew.gui;

import de.renew.engine.searcher.Occurrence;
import de.renew.engine.searcher.OccurrenceDescription;
import de.renew.engine.searcher.Searcher;
import de.renew.engine.searcher.VariableMapperCopier;

import de.renew.net.TransitionInstance;

import de.renew.unify.Impossible;

import java.util.Collection;
import java.util.Collections;


/**
 * Occurrence of a breakpoint inscription.
 *
 * BreakpointOccurrence.java
 * Created: Wed May 31  2000
 * @author Michael Duvigneau
 *
 * @see BreakpointInscription
 **/
class BreakpointOccurrence implements Occurrence {
    private GlobalTransitionBreakpoint breakpoint;

    /**
     * The transition instance this inscription occurs at.
     * The breakpoint has to know at which transition instance
     * it occurrs to be able to inform the user.
     **/
    private TransitionInstance instance;

    BreakpointOccurrence(GlobalTransitionBreakpoint breakpoint,
                         TransitionInstance instance) {
        this.breakpoint = breakpoint;
        this.instance = instance;
    }

    /**
     * @return    empty enumeration
     *            (breakpoint holds in all bindings)
     * @exception de.renew.unify.Impossible
     *            never?
     */
    public Collection makeBinders(Searcher searcher) throws Impossible {
        return Collections.EMPTY_SET;
    }

    /**
     * @return one executable of this breakpoint
     * @see BreakpointExecutable
     **/
    public Collection makeExecutables(VariableMapperCopier copier) {
        return Collections.singleton(new BreakpointExecutable(breakpoint,
                                                              instance));
    }

    public OccurrenceDescription makeOccurrenceDescription(VariableMapperCopier copier) {
        return null;
    }
}