/*
 * Created on 16.05.2003
 *
 */
package de.renew.gui;

import de.renew.application.SimulatorPlugin;

import de.renew.plugin.PluginProperties;
import de.renew.plugin.PropertyHelper;

import java.awt.Component;

import java.util.Properties;


/**
 * An option panel controller to configure concurrent features of
 * the simulation engine.
 * @author Michael Duvigneau
 **/
public class ConfigureEngineController
        implements ConfigureSimulationTabController {
    private ConfigureEngineTab tab;

    public ConfigureEngineController() {
        this.tab = new ConfigureEngineTab(this);
    }

    public Component getTab() {
        return tab;
    }

    public void commitTab(Properties props) {
        boolean sequential = tab.getSequential();
        int mult = Math.abs(Integer.parseInt(tab.getMultiplicity()));
        if (mult == 0) {
            mult = 1;
        }
        if (sequential) {
            mult = -mult;
        }
        if (mult == 1) {
            props.remove(SimulatorPlugin.MODE_PROP_NAME);
            if (PluginProperties.debug()) {
                System.out.println("ConfigureEngineController: Configured simulatorMode=null.");
            }
        } else {
            props.setProperty(SimulatorPlugin.MODE_PROP_NAME,
                              Integer.toString(mult));
            if (PluginProperties.debug()) {
                System.out.println("ConfigureEngineController: Configured simulatorMode="
                                   + mult + ".");
            }
        }

        boolean classReinit = tab.getClassReinit();
        props.setProperty(SimulatorPlugin.REINIT_PROP_NAME,
                          Boolean.toString(classReinit));
        if (PluginProperties.debug()) {
            System.out.println("ConfigureEngineController: Configured classReinit="
                               + classReinit + ".");
        }

        boolean eagerSimulation = tab.getEagerSimulation();
        props.setProperty(SimulatorPlugin.EAGER_PROP_NAME,
                          Boolean.toString(eagerSimulation));
        if (PluginProperties.debug()) {
            System.out.println("ConfigureEngineController: Configured eagerSimulation="
                               + eagerSimulation + ".");
        }
    }

    public void updateTab(Properties props) {
        int mult = PropertyHelper.getIntProperty(props,
                                                 SimulatorPlugin.MODE_PROP_NAME,
                                                 1);
        boolean sequential = (mult < 0);
        if (sequential) {
            mult = -mult;
        }
        tab.setSequential(sequential);
        tab.setMultiplicity(Integer.toString(mult));

        boolean classReinit = PropertyHelper.getBoolProperty(props,
                                                             SimulatorPlugin.REINIT_PROP_NAME);
        tab.setClassReinit(classReinit);

        boolean eagerSimulation = PropertyHelper.getBoolProperty(props,
                                                                 SimulatorPlugin.EAGER_PROP_NAME);
        tab.setEagerSimulation(eagerSimulation);
    }
}