package de.renew.gui;

import de.renew.net.NetNotFoundException;

import de.renew.plugin.PluginProperties;

import de.renew.shadow.DefaultShadowNetLoader;
import de.renew.shadow.ShadowNet;
import de.renew.shadow.ShadowNetLoader;
import de.renew.shadow.ShadowNetSystem;

import java.util.Properties;


/**
 * A shadow net loader implementation which first delegates to a
 * {@link DefaultShadowNetLoader} and then to the gui's
 * {@link CPNDrawingLoader}.
 * <p>
 * </p>
 * GuiShadowNetLoader.java
 * Created: Tue Dec  5  2001
 * @author Michael Duvigneau
 **/
class GuiShadowNetLoader implements ShadowNetLoader {
    private ShadowNetLoader shadowLoader;
    private CPNDrawingLoader drawingLoader;

    public GuiShadowNetLoader(CPNDrawingLoader drawingLoader, Properties props) {
        this.shadowLoader = new DefaultShadowNetLoader(props);
        this.drawingLoader = drawingLoader;
        if (props != null) {
            drawingLoader.configureNetPath(props);
        }
    }

    public ShadowNet loadShadowNet(String netName, ShadowNetSystem netSystem)
            throws NetNotFoundException {
        try {
            return shadowLoader.loadShadowNet(netName, netSystem);
        } catch (NetNotFoundException e) {
            // So there is no matching shadow net file.
            // Let's look for drawing files...
        }

        if (PluginProperties.debug()) {
            System.out.println("GuiShadowNetLoader: Asking for drawing "
                               + netName + ".");
        }
        CPNDrawing drawing = drawingLoader.getDrawing(netName);
        if (drawing != null) {
            if (PluginProperties.debug()) {
                System.out.println("GuiShadowNetLoader: Trying to compile drawing "
                                   + drawing + ".");
            }
            try {
                ShadowNet net = drawing.buildShadow(netSystem);
                if (net != null) {
                    if (PluginProperties.debug()) {
                        System.out.println("GuiShadowNetLoader: Successful.");
                    }
                    return net;
                }
            } catch (Exception e) {
                if (PluginProperties.debug()) {
                    System.out.println("GuiShadowNetLoader: Problem " + e + ".");
                }
                throw new NetNotFoundException(netName, e);
            }
        } else {
            if (PluginProperties.debug()) {
                System.out.println("GuiShadowNetLoader: No drawing found.");
            }
        }
        throw new NetNotFoundException(netName);
    }
}