package de.renew.gui;

import CH.ifa.draw.DrawPlugin;

import CH.ifa.draw.application.MenuManager;

import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginException;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;
import de.renew.plugin.command.CLCommand;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.net.URL;

import java.util.Map;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;


/**
 * @author 6schumac
 *
 */
public class GuiPromptPlugin extends PluginAdapter {
    // MenuExtender menuExtender;
    JMenuItem _menu;
    PromptDialog _promptDialog;

    public GuiPromptPlugin(URL location) throws PluginException {
        super(location);
    }

    public GuiPromptPlugin(PluginProperties props) {
        super(props);
    }

    public void init() {
        _menu = createMenu();
        DrawPlugin.getCurrent().getMenuManager().registerMenu(DrawPlugin.PLUGINS_MENU,
                                                              _menu);


        // menuExtender = new GuiPromptMenuExtender();
        // GuiPlugin.getCurrent().getMenuHolder().addMenu(_menu);
    }

    public boolean cleanup() {
        DrawPlugin current = DrawPlugin.getCurrent();

        if (current != null) {
            // current.removeMenuExtender(menuExtender);
            current.getMenuManager().unregisterMenu(_menu);
        }
        if (_promptDialog != null) {
            _promptDialog.dispose();
            _promptDialog = null;
        }
        return true;
    }

    private class PromptDialog extends JFrame {
        JTextField _inputField = new JTextField(30);

        public PromptDialog() {
            super("command input");

            JButton execute = new JButton("execute");
            JButton clear = new JButton("clear");

            execute.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        executeCommand();
                    }
                });
            clear.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        _inputField.setText("");
                    }
                });

            JPanel buttonPanel = new JPanel();
            buttonPanel.add(execute);
            buttonPanel.add(clear);

            JRootPane root = getRootPane();
            root.setLayout(new BorderLayout());
            root.add(_inputField, BorderLayout.NORTH);
            root.add(buttonPanel, BorderLayout.CENTER);

            addWindowListener(new WindowAdapter() {
                    public void windowClosing(WindowEvent e) {
                        System.out.println("window closing.");
                        _promptDialog = null;
                        dispose();
                    }
                });
            _inputField.addKeyListener(new KeyAdapter() {
                    public void keyPressed(KeyEvent e) {
                        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                            executeCommand();
                        }
                    }
                });

            pack();
        }

        private void executeCommand() {
            try {
                String input = _inputField.getText();
                Map commands = PluginManager.getInstance().getCLCommands();
                String[] cl = splitString(input);
                if (cl.length == 0) {
                    return;
                }
                final CLCommand c = (CLCommand) commands.get(cl[0]);
                if (c == null) {
                    System.out.println("GuiPrompt: unknown command: " + cl[0]);
                } else {
                    final String[] nc = new String[cl.length - 1];
                    for (int i = 0; i < nc.length; i++) {
                        nc[i] = cl[i + 1];
                    }
                    if (PluginProperties.debug()) {
                        System.out.println("GuiPrompt: scheduling command: "
                                           + input);
                    }
                    new Thread() {
                            public void run() {
                                c.execute(nc);
                            }
                        }.start();
                }
            } catch (RuntimeException e) {
                System.out.println("GuiPrompt: an exeption occurred: " + e);
                e.printStackTrace();
            }
        }

        private String[] splitString(String s) {
            StringTokenizer st = new StringTokenizer(s);
            String[] result = new String[st.countTokens()];
            int i = 0;
            while (st.hasMoreTokens()) {
                result[i++] = st.nextToken();
            }
            return result;
        }

        public void show() {
            super.show();
            registerExitBlock();
        }

        public void dispose() {
            super.dispose();
            registerExitOk();
        }
    }

    private JMenu createMenu() {
        JMenu menu = new JMenu("Gui Prompt");
        JMenuItem mi = new JMenuItem("show prompt");
        mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (_promptDialog == null) {
                        _promptDialog = new PromptDialog();
                    }
                    _promptDialog.show();
                }
            });
        menu.add(mi);
        menu.putClientProperty(MenuManager.ID_PROPERTY, getProperty("requires"));
        return menu;
    }
}