package de.renew.plugin;

import de.renew.plugin.command.CLCommand;

import java.net.URL;
import java.net.URLClassLoader;


/**
 * This ClassLoader is responsible for loading the plugin classes.
 * It allows adding classpath items dynamically.
 * It provides a Singleton access interface to ensure that only one
 * PluginClassLoader is in use.
 */
public class PluginClassLoader extends URLClassLoader {
    private static PluginClassLoader _loader;

    /**
     * Creates an instance of the PluginClassLoader and stores it in the
     * _loader Singleton member.
     * @exception RuntimeException is thrown if a PluginClassLoader has previously
     * been instantiated .
     */
    protected PluginClassLoader(URL[] urls, ClassLoader parent) {
        super(urls, parent);

        if (_loader == null) {
            _loader = this;
        } else {
            throw new RuntimeException("PluginClassLoader: only one instance of this class is allowed.");
        }
    }

    public PluginClassLoader(ClassLoader parent) {
        super(new URL[0], parent);
    }

    /**
     * Add the given URL to the list where classes are sought for.
     */
    public void addURL(URL url) {
        if (PluginProperties.debug()) {
            System.out.println(this + " adding URL " + url);
        }
        super.addURL(url);
    }

    class PackageCountCommand implements CLCommand {
        /*
         * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[])
         */
        public void execute(String[] args) {
            String prefix = "";
            if (args.length != 0) {
                prefix = args[0];
            }
            Package[] packages = Package.getPackages();
            int count = 0;
            for (int i = 0; i < packages.length; i++) {
                if (packages[i].getName().startsWith(prefix)) {
                    System.out.println(packages[i]);
                    count++;
                }
            }
            System.out.println(count + " packages found.");
        }

        /*
         * @see de.renew.plugin.command.CLCommand#getDescription()
         */
        public String getDescription() {
            return "prints the number of packages in the class loader";
        }
    }
}