package de.renew.plugin.command;

import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * This command loads a plugin from the given URL.
 * If the argument "-v" is given, verbose output is printed.
 *
 * @author J&ouml;rn Schumacher
 *
 */
public class LoadCommand implements CLCommand {
    public void execute(String[] args) {
        boolean verbose = false;
        URL url = null;
        if (args.length != 1) {
            System.err.println("usage: load <url>");
        } else {
            System.out.println("looking for plugin at " + args[0] + " ...");
            try {
                url = new URL(args[0]);

                IPlugin plugin = PluginManager.getInstance().loadPlugin(url);
                System.out.println("loaded plugin: " + plugin.getName() + ".");
            } catch (MalformedURLException e) {
                System.out.println("Malformed URL: " + e.getMessage());
            } catch (Exception e) {
                System.out.println("load failed: " + e);
                e.printStackTrace();
            }
        }
    }

    public String getDescription() {
        return "loads a new plugin given a URL";
    }
}