package de.renew.plugin.command;

import de.renew.plugin.PluginManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;


/**
 * This command reads the file given in the first argument
 * and interprets eachs line as a command to be executed by the PluginManager.
 *
 * @author J&ouml;rn Schumacher
 */
public class ScriptCommand implements CLCommand {

    /**
     * @see de.renew.plugin.command.CLCommand#execute(String[])
     */
    public void execute(String[] args) {
        if (args.length > 1) {
            System.out.println("ScriptCommand: more than one parameter given: using only "
                               + args[0]);
        }
        Collection lines = readFile(args[0]);
        Collection commands = createCommands(lines);
        Iterator commandIterator = commands.iterator();
        while (commandIterator.hasNext()) {
            try {
                CommandArgumentTuple command = (CommandArgumentTuple) commandIterator
                                               .next();
                command.execute();
            } catch (RuntimeException e) {
                System.out.println("PluginManager.PromptThread: an exeption occurred: "
                                   + e);
                e.printStackTrace();
            }
        }
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    public String getDescription() {
        return "reads an input file and interprets every line as a PluginManager command";
    }

    private Collection readFile(String filename) {
        Vector lines = new Vector();
        File f = new File(filename);
        String line = "";
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(f)));
            while (line != null) {
                line = reader.readLine();
                if (line != null) {
                    lines.add(line);
                }
            }
        } catch (FileNotFoundException e) {
            System.out.println("ScriptCommand: file " + filename
                               + " not found.");
        } catch (IOException e) {
            System.out.println("ScriptCommand: exception when reading "
                               + filename + ": " + e);
            e.printStackTrace();
        }
        return lines;
    }

    private Collection createCommands(Collection toConvertCollection) {
        Map commands = PluginManager.getInstance().getCLCommands();
        Vector result = new Vector();
        Iterator toConvertIterator = toConvertCollection.iterator();
        while (toConvertIterator.hasNext()) {
            String toConvert = (String) toConvertIterator.next();
            String[] split = splitString(toConvert);
            if ((split.length == 0) || (split[0].trim().startsWith("#"))) {
                // So this is an empty line or a comment.
                // Ignore it.
            } else {
                CLCommand c = (CLCommand) commands.get(split[0]);
                if (c == null) {
                    System.out.println("ScriptCommand: unknown command: "
                                       + split[0]);
                } else {
                    String[] nc = new String[split.length - 1];
                    for (int i = 0; i < nc.length; i++) {
                        nc[i] = split[i + 1];
                    }
                    CommandArgumentTuple cat = new CommandArgumentTuple(c, nc);
                    result.add(cat);
                }
            }
        }
        return result;
    }

    private String[] splitString(String s) {
        StringTokenizer st = new StringTokenizer(s);
        String[] result = new String[st.countTokens()];
        int i = 0;
        while (st.hasMoreTokens()) {
            result[i++] = st.nextToken();
        }
        return result;
    }

    private class CommandArgumentTuple {
        private CLCommand _command;
        private String[] _arguments;

        public CommandArgumentTuple(CLCommand command, String[] args) {
            _command = command;
            _arguments = args;
        }

        public void execute() {
            _command.execute(_arguments);
        }
    }
}