package de.renew.engine.common;

import de.renew.engine.searcher.ChannelBinder;
import de.renew.engine.searcher.Occurrence;
import de.renew.engine.searcher.OccurrenceDescription;
import de.renew.engine.searcher.Searcher;
import de.renew.engine.searcher.VariableMapperCopier;

import de.renew.expression.Expression;
import de.renew.expression.VariableMapper;

import de.renew.unify.Impossible;
import de.renew.unify.Variable;

import java.util.Collection;
import java.util.Collections;


public class DownlinkOccurrence implements Occurrence {
    private Expression params;
    private Expression callee;
    private String name;
    private boolean isOptional;
    private VariableMapper mapper;

    public DownlinkOccurrence(Expression params, Expression callee,
                              String name, boolean isOptional,
                              VariableMapper mapper) {
        this.params = params;
        this.callee = callee;
        this.name = name;
        this.isOptional = isOptional;
        this.mapper = mapper;
    }

    public Collection makeBinders(Searcher searcher) throws Impossible {
        Variable calleeVariable = new Variable(callee.startEvaluation(mapper,
                                                                      searcher.recorder,
                                                                      searcher.calcChecker),
                                               searcher.recorder);
        Variable paramsVariable = new Variable(params.startEvaluation(mapper,
                                                                      searcher.recorder,
                                                                      searcher.calcChecker),
                                               searcher.recorder);

        return Collections.singleton(new ChannelBinder(calleeVariable, name,
                                                       paramsVariable,
                                                       isOptional));
    }

    public Collection makeExecutables(VariableMapperCopier copier) {
        // A downlink does not cause any explicit action
        // during execution.
        return Collections.EMPTY_SET;
    }

    public OccurrenceDescription makeOccurrenceDescription(VariableMapperCopier variableMapperCopier) {
        return null;
    }
}