package de.renew.net.arc;

import de.renew.engine.searcher.EarlyExecutable;
import de.renew.engine.searcher.Tracer;

import de.renew.net.PlaceInstance;

import de.renew.unify.Impossible;


class TestArcExecutable implements EarlyExecutable {
    PlaceInstance placeInstance;
    Object token;
    boolean trace;
    boolean releaseImmediately;

    TestArcExecutable(PlaceInstance placeInstance, Object token,
                      boolean releaseImmediately, boolean trace) {
        this.placeInstance = placeInstance;
        this.token = token;
        this.trace = trace;


        // If releaseImmediately is false, the tested token will remain
        // tested until it is explicitly untested.
        this.releaseImmediately = releaseImmediately;
    }

    public long lockPriority() {
        return placeInstance.lockOrder;
    }

    public int phase() {
        return TEST;
    }

    public void lock() {
        placeInstance.lock.lock();
    }

    public void verify() throws Impossible {
        placeInstance.testToken(token);
    }

    public void execute(Tracer tracer) {
        if (releaseImmediately) {
            if (trace) {
                tracer.trace("    Quickly Testing " + token + " in "
                             + placeInstance);
            }
            placeInstance.untestToken(token);
        } else {
            if (trace) {
                tracer.trace("    Testing " + token + " in " + placeInstance);
            }
        }
    }

    public void rollback() {
        placeInstance.untestToken(token);
    }

    public void unlock() {
        placeInstance.lock.unlock();
    }
}