package de.renew.remote;

import de.renew.engine.simulator.Binding;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;


/**
 * Implements the BindingAccessor interface.
 * The binding accessor wraps bindings to provide the
 * functionality to list and fire bindings from the client.
 *
 * @author Thomas Jacob
 */
public class BindingAccessorImpl extends UnicastRemoteObject
        implements BindingAccessor {

    /**
     * The object for the accessor.
     */
    protected final Binding binding;

    /**
     * Creates a new BindingAccessor.
     * @param binding The binding for the accessor.
     * @exception java.rmi.RemoteException if an RMI failure occured.
     */
    public BindingAccessorImpl(Binding binding) throws RemoteException {
        super(0, SocketFactoryDeterminer.getInstance(),
              SocketFactoryDeterminer.getInstance());
        this.binding = binding;
    }

    /**
     * Executes the binding.
     * @param tracer The tracer to output messages to.
     * @param asynchronous Whether to execute the binding asynchronously.
     * @exception java.rmi.RemoteException if an RMI failure occured.
     */
    public boolean execute(TracerAccessor tracer, boolean asynchronous)
            throws RemoteException {
        return binding.execute(new RemoteTracer(tracer), asynchronous);
    }

    /**
     * Returns the description of the binding.
     * @return The description.
     * @exception java.rmi.RemoteException if an RMI failure occured.
     */
    public String getDescription() throws RemoteException {
        return binding.getDescription();
    }
}