package de.renew.shadow;

import de.renew.net.Net;
import de.renew.net.NetLoader;


public interface ShadowCompiler {

    /**
     * Set the net loader that this compiler uses when a
     * net is found missing amidst compiling.
     *
     * @param loopbackNetLoader the net loader
     */
    public abstract void setLoopbackNetLoader(NetLoader loopbackNetLoader);

    /**
     * Set the shadow lookup that will be used for lookups during
     * compilation and that will receive the compilation results.
     *
     * @param shadowLookup the lookup
     */
    public abstract void setShadowLookup(ShadowLookup shadowLookup);

    /**
     * Create one net of the appropriate class, using the given
     * name as the net's name. This method is called before any
     * preprocessors have run and before the compilation has been
     * started.
     *
     * @param name the net's name
     * @return a net with the correct name and class
     */
    public Net createNet(String name);

    /**
     * Get the list of preprocessors that must run before this compiler
     * can start the compilation.
     *
     * @return the preprocessors in an array or the empty array,
     *   if no preprocessors must run
     */
    public ShadowPreprocessor[] getRequiredPreprocessors();

    public abstract void compile(ShadowNet shadowNet) throws SyntaxException;

    public String checkDeclarationNode(String inscr, boolean special,
                                       ShadowNet shadowNet)
            throws SyntaxException;

    public String checkArcInscription(String inscr, boolean special,
                                      ShadowNet shadowNet)
            throws SyntaxException;

    public String checkTransitionInscription(String inscr, boolean special,
                                             ShadowNet shadowNet)
            throws SyntaxException;

    public String checkPlaceInscription(String inscr, boolean special,
                                        ShadowNet shadowNet)
            throws SyntaxException;
}